-- AutomaticBeaconLights Configuration
-- Customize the behavior of beacon lights by setting BeaconControl to 1, 2, or 3:
-- 1: Automatic beacons for player actions only (entering, driving, exiting with engine on). AI behavior unchanged.
-- 2: Automatic beacons for AI workers only (on when AI controls vehicle, off when AI stops or engine off). Player behavior unchanged.
-- 3: (Default) Full control - beacons on for both player and AI when vehicle is occupied or engine is running.
local BeaconControl = 3  -- Default setting. Change to 1 or 2 to adjust behavior.

-- Enable or disable detailed debug logging:
-- 0: (Default) Disabled - no log output from this mod.
-- 1: Enabled - full debug output including state changes and events.
local DebuggingEnabled = 0  -- Default setting. Change to 1 for detailed logs.

AutomaticBeaconLights = {}

function AutomaticBeaconLights.prerequisitesPresent(specializations)
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: prerequisitesPresent called") end
    return true
end

function AutomaticBeaconLights.registerEventListeners(vehicleType)
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Registering event listeners for " .. vehicleType.name) end
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onEnterVehicle", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", AutomaticBeaconLights)
end

function AutomaticBeaconLights:onLoad()
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onLoad called - Script version with AI fieldwork fix and deletion fail-safe") end
    self.beaconsActive = false
    self.wasAIActive = false
    self.wasPlayerActive = false
    self.wasUnderAIControl = false
    self.isAIWorking = false
    self.isValid = true
end

function AutomaticBeaconLights:onDelete()
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onDelete called - Vehicle is being removed") end
    self.isValid = false
end

function AutomaticBeaconLights:onUpdate(dt)
    if not self.isValid or self.getIsActive == nil then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Skipping update - Vehicle is invalid or deleted") end
        return
    end

    local isActive = self:getIsActive()
    local isPlayerEntered = self:getIsEntered()
    local isAIActive = self:getIsAIActive()
    local isMotorStarted = self.getIsMotorStarted and self:getIsMotorStarted() or false

    if isAIActive or (self.wasUnderAIControl and isMotorStarted) then
        self.isAIWorking = true
    elseif not isMotorStarted then
        self.isAIWorking = false
    end

    if DebuggingEnabled == 1 then
        print("AutomaticBeaconLights: Debug - Active: " .. tostring(isActive) .. ", MotorStarted: " .. tostring(isMotorStarted) .. ", Player: " .. tostring(isPlayerEntered) .. ", AI: " .. tostring(isAIActive) .. ", WasUnderAI: " .. tostring(self.wasUnderAIControl) .. ", AIWorking: " .. tostring(self.isAIWorking))
    end

    if isAIActive then self.wasAIActive = true; self.wasUnderAIControl = true end
    if isPlayerEntered then self.wasPlayerActive = true end

    local shouldBeaconsBeOn = false
    if BeaconControl == 1 then
        shouldBeaconsBeOn = isPlayerEntered or (self.wasPlayerActive and isMotorStarted)
    elseif BeaconControl == 2 then
        shouldBeaconsBeOn = isAIActive or (self.isAIWorking and isMotorStarted)
    elseif BeaconControl == 3 then
        shouldBeaconsBeOn = isMotorStarted or isPlayerEntered or isAIActive or self.isAIWorking
    end

    if DebuggingEnabled == 1 then
        print("AutomaticBeaconLights: State - Active: " .. tostring(isActive) .. ", Player: " .. tostring(isPlayerEntered) .. ", AI: " .. tostring(isAIActive) .. ", WasAI: " .. tostring(self.wasAIActive) .. ", WasPlayer: " .. tostring(self.wasPlayerActive) .. ", MotorStarted: " .. tostring(isMotorStarted) .. ", ShouldBeOn: " .. tostring(shouldBeaconsBeOn) .. ", BeaconsActive: " .. tostring(self.beaconsActive))
    end

    if shouldBeaconsBeOn then
        self:setBeaconLightsVisibility(true)
        self.beaconsActive = true
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Forcing beacons on (Player: " .. tostring(isPlayerEntered) .. ", AI: " .. tostring(isAIActive) .. ", AIWorking: " .. tostring(self.isAIWorking) .. ")") end
    elseif not shouldBeaconsBeOn and self.beaconsActive then
        self:setBeaconLightsVisibility(false)
        self.beaconsActive = false
        self.wasAIActive = false
        self.wasPlayerActive = false
        self.wasUnderAIControl = false
        self.isAIWorking = false
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Deactivating beacons") end
    end
end

function AutomaticBeaconLights:onEnterVehicle()
    if not self.isValid then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Skipping onEnterVehicle - Vehicle is invalid") end
        return
    end
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onEnterVehicle called") end
    if self.wasUnderAIControl and not self:getIsAIActive() then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Resetting post-AI control") end
        self.beaconsActive = false
    end
end

function AutomaticBeaconLights:onLeaveVehicle()
    if not self.isValid then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Skipping onLeaveVehicle - Vehicle is invalid") end
        return
    end
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onLeaveVehicle called") end
end